"""
# Simulation settings

Main simulation settings for the SWAP model.

Modules:
    general: General simulation settings for the SWAP model.
    metadata: Metadata for the SWAP model.
    richards: Richards simulation settings for the SWAP model.
"""
from .metadata import *
from .general import *
from .richards import *
