"""
# Boundary subpackage

This subpackage deals with boundary conditions for the SWAP model.

Modules:
    bottomboundary: Bottom boundary condition settings for the SWAP model.
"""
from .boundary import BottomBoundary
