"""
# Atmosphere subpackage

Meteorological settings and data for SWAP simulations.

Modules:
    metfile: meteorological data for SWAP simulations
    meteorology: meteorological settings for SWAP simulations
___________________________________________________________________________
"""

from .meteorology import *
from .metfile import *
