"""Default values for the Richards' equation"""

DERAULT_RICHARDS = {
    'dtmin': 1.0e-7,
    'dtmax': 0.1,
    'gwlconv': 20.0,
    'critdevh1cp': 1.0e-2,
    'critdevh2cp': 1.0e-1,
    'critdevponddt': 1.0e-4,
    'maxit': 30,
    'maxbacktr': 10,
    'swkmean': 1,
    'swkimpl': 0
}
